/* -----------------------------------------------------------------------------
 *
 * Giada - Your Hardcore Loopmachine
 *
 * -----------------------------------------------------------------------------
 *
 * Copyright (C) 2010-2017 Giovanni A. Zuliani | Monocasual
 *
 * This file is part of Giada - Your Hardcore Loopmachine.
 *
 * Giada - Your Hardcore Loopmachine is free software: you can
 * redistribute it and/or modify it under the terms of the GNU General
 * Public License as published by the Free Software Foundation, either
 * version 3 of the License, or (at your option) any later version.
 *
 * Giada - Your Hardcore Loopmachine is distributed in the hope that it
 * will be useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Giada - Your Hardcore Loopmachine. If not, see
 * <http://www.gnu.org/licenses/>.
 *
 * -------------------------------------------------------------------------- */



#ifndef G_CHANNEL_H
#define G_CHANNEL_H


#include <vector>
#include <string>
#include <pthread.h>
#include "midiMapConf.h"
#include "recorder.h"

#ifdef WITH_VST
	#include "../deps/juce-config.h"
#endif


class Plugin;
class MidiMapConf;
class geChannel;


class Channel
{
protected:

	

	void sendMidiLmessage(uint32_t learn, const giada::m::midimap::message_t& msg);

	

	float calcPanning(int ch);

#ifdef WITH_VST

	

	juce::MidiBuffer midiBuffer;

#endif

	

	int bufferSize;

  

  int midiFilter;

	

	int previewMode;

	float pan;
	bool armed;
	std::string name;

public:

	Channel(int type, int status, int bufferSize);

	virtual ~Channel();

	

	virtual void copy(const Channel* src, pthread_mutex_t* pluginMutex) = 0;

	

	virtual int readPatch(const std::string& basePath, int i,
    pthread_mutex_t* pluginMutex, int samplerate, int rsmpQuality);

	

	virtual void process(float* outBuffer, float* inBuffer) = 0;

	

	virtual void preview(float* outBuffer) = 0;

	

	virtual void start(int frame, bool doQuantize, int quantize,
			bool mixerIsRunning, bool forceStart, bool isUserGenerated) = 0;

	

	virtual void stop() = 0;

	

	virtual void kill(int frame) = 0;

	

	virtual void setMute  (bool internal) = 0;
	virtual void unsetMute(bool internal) = 0;

	

	virtual void empty() = 0;

	

	virtual void stopBySeq(bool chansStopOnSeqHalt) = 0;

	

	virtual void quantize(int index, int localFrame) = 0;

	

	virtual void onZero(int frame, bool recsStopOnChanHalt) = 0;

	

	virtual void onBar(int frame) = 0;

	

	 

	virtual void parseAction(giada::m::recorder::action* a, int localFrame,
    int globalFrame, int quantize, bool mixerIsRunning) = 0;

	

	virtual void rewind() = 0;

	

	virtual void clear() = 0;

	

	virtual bool canInputRec() = 0;

	

	virtual int writePatch(int i, bool isProject);

	

	virtual void receiveMidi(uint32_t msg);

	

	virtual bool allocBuffers();

	

	bool isPlaying() const;

	

	void sendMidiLmute();
	void sendMidiLsolo();
	void sendMidiLplay();

	void setPan(float v);
	float getPan() const;

	void setArmed(bool b);
	bool isArmed() const;

	std::string getName() const;
	void setName(const std::string& s);

	void setPreviewMode(int m);
	bool isPreview() const;

#ifdef WITH_VST

	

	juce::MidiBuffer& getPluginMidiEvents();

	void clearMidiBuffer();

#endif

	int    index;                 
	int    type;                  
	int    status;                
	int    key;                   
	float  volume;                
	float  volume_i;              
	float  volume_d;              
	bool   mute_i;                
	bool 	 mute_s;                
	bool   mute;                  
	bool   solo;
  bool   hasActions;            
  bool   readActions;           
	int 	 recStatus;             
	float* vChan;                 
  geChannel* guiChannel;        

	

  bool     midiIn;              
  uint32_t midiInKeyPress;
  uint32_t midiInKeyRel;
  uint32_t midiInKill;
  uint32_t midiInArm;
  uint32_t midiInVolume;
  uint32_t midiInMute;
  uint32_t midiInSolo;

	

	bool     midiOutL;
  uint32_t midiOutLplaying;
  uint32_t midiOutLmute;
  uint32_t midiOutLsolo;

#ifdef WITH_VST
  std::vector <Plugin*> plugins;
#endif

};


#endif
