/* -----------------------------------------------------------------------------
 *
 * Giada - Your Hardcore Loopmachine
 *
 * -----------------------------------------------------------------------------
 *
 * Copyright (C) 2010-2017 Giovanni A. Zuliani | Monocasual
 *
 * This file is part of Giada - Your Hardcore Loopmachine.
 *
 * Giada - Your Hardcore Loopmachine is free software: you can
 * redistribute it and/or modify it under the terms of the GNU General
 * Public License as published by the Free Software Foundation, either
 * version 3 of the License, or (at your option) any later version.
 *
 * Giada - Your Hardcore Loopmachine is distributed in the hope that it
 * will be useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Giada - Your Hardcore Loopmachine. If not, see
 * <http://www.gnu.org/licenses/>.
 *
 * -------------------------------------------------------------------------- */



#ifndef GE_WAVEFORM_H
#define GE_WAVEFORM_H


#include <vector>
#include <FL/Fl_Widget.H>


class SampleChannel;


class geWaveform : public Fl_Widget
{
private:

	static const int FLAG_WIDTH  = 20;
	static const int FLAG_HEIGHT = 20;
	static const int BORDER      = 8;  
	static const int SNAPPING    = 16;

	

	struct
	{
		int a;
		int b;
	} m_selection;

	

	struct
	{
		int* sup;   
		int* inf;   
		int  size;  
	} m_data;

	struct
	{
		bool snap;
		int level;
		std::vector<int> points;
	} m_grid;

	SampleChannel* m_ch;
	int m_chanStart;
	bool m_chanStartLit;
	int m_chanEnd;
	bool m_chanEndLit;
	bool m_pushed;
	bool m_dragged;
	bool m_resizedA;
	bool m_resizedB;
	float m_ratio;
	int m_mouseX;
	int m_mouseY;

	

	bool mouseOnStart();
	bool mouseOnEnd();

	

	bool mouseOnSelectionA();
	bool mouseOnSelectionB();

	int pixelToFrame(int p);  
	int frameToPixel(int f);  

	

	void fixSelection();

	

	void freeData();

	

	bool smaller();

	

	int snap(int pos);

	

	void drawSelection();
	void drawWaveform(int from, int to);
	void drawGrid(int from, int to);
	void drawStartEndPoints();
	void drawPlayHead();

	void selectAll();

public:

	static const int ZOOM_IN  = -1;
	static const int ZOOM_OUT = 0;

	geWaveform(int x, int y, int w, int h, SampleChannel* ch, const char* l=0);
	~geWaveform();

	void draw() override;
	int  handle(int e) override;

	

	int alloc(int datasize, bool force=false);

	

	void recalcPoints();

	

	void setZoom(int type);

	

	void stretchToWindow();

	

	void refresh();

	

	void setGridLevel(int l);

	void setSnap(bool v);
	bool getSnap();
	int getSize();

	

	bool isSelected();

	int getSelectionA();
	int getSelectionB();

	

	void clearSel();
};


#endif
